#   make the Isotopes.plist file
#  data from:  https://physics.nist.gov/cgi-bin/Compositions/stand_alone.pl?ele=&all=all&ascii=ascii2&isotype=all
import urllib
nist = 'https://physics.nist.gov/cgi-bin/Compositions/stand_alone.pl?ele=&all=all&ascii=ascii2&isotype=all'



def isoBlock2dict(isoBlock):
	lines = isoBlock.split('\n')
	iso = {}
	for line in lines:
		keyVal = line.split(' = ')

		if keyVal[0] == 'Atomic Number':
			iso['Z'] = int(keyVal[1])
		elif keyVal[0] == 'Atomic Symbol':
			iso['symbol'] = keyVal[1]
		elif keyVal[0] == 'Mass Number':
			iso['massNumber'] = int(keyVal[1])
		elif keyVal[0] == 'Relative Atomic Mass':
			ll = keyVal[1].split('(')
			try:	iso['amu'] = float(ll[0])
			except:	iso['amu'] = float('nan')
		elif keyVal[0] == 'Isotopic Composition':
			ll = keyVal[1].split('(')
			try:	iso['natural'] = float(ll[0])
			except:	iso['natural'] = 0.0
		elif keyVal[0] == 'Standard Atomic Weight':
			iso['stdWeights'] = keyVal[1]
		elif keyVal[0] == 'Notes':
			iso['note'] = keyVal[1]
	return iso



def oneZisos(all,Z):
	# find all isotopes for one Z, and combine into a list of tuples with (massNumber, amu, natural abundance)
	Ziso = []
	for iso in all:
		if iso['Z'] == Z:
			symbol = iso['symbol']
			amu = iso['amu']
			massNumber = iso['massNumber']
			natural = iso['natural']
			Ziso.append((massNumber,amu,natural))

	return (symbol,Ziso)



def oneZ2xmlStr(oneZ):
	# for one element, take output from oneZisos(), and make a string for the plist file

	out = '\n<Z'+str(Z)+'>\n'
	for iso in oneZ[1]:
		if iso[2] == 0.0:
			natural = '0'
		else:
			natural = str(iso[2])
		out += '\t<is>'+str(iso[0])+' '+str(iso[1])+' '+natural+'</is>\n'
	out += '</Z'+str(Z)+'>\n'
	return out




# get data from web
print 'loading isotope data from ',nist
fin = urllib.urlopen(nist)
buf = fin.read()
isoBlocks = buf.split("\n\n")

# parse out each isotope from text into a list of dicts
all = []
for block in isoBlocks:
	iso = isoBlock2dict(block)
	if len(iso) == 7:
		all.append(isoBlock2dict(block))

# open xml file, and write to it
fout = open('isotopes New.xml','w' )
header  = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n\n"
header += "<isotopes xmlns=\"http://sector34.xray.aps.anl.gov/34ide:isotopes\">\n"
header += "<!-- contents of <is>:  MassNumber AtomicMass NaturalFraction -->\n"

fout.write(header)

cnt = 0
for Z in range(1,119):
#	print 'writing Z =',Z
	fout.write(oneZ2xmlStr(oneZisos(all,Z)))
	cnt =+ 1

fout.write("\n</isotopes>\n")
fout.close()

print 'done'
